@@ -94,6 +94,14 @@ create_thumbnail_frame (int        width,
 	return retval;
 }
 
+/**
+ * ev_document_misc_get_thumbnail_frame:
+ * @width: the desired width
+ * @height: the desired height
+ * @source_pixbuf: a #GdkPixbuf
+ *
+ * Returns: (transfer full): a #GdkPixbuf
+ */
 GdkPixbuf *
 ev_document_misc_get_thumbnail_frame (int        width,
 				      int        height,
@@ -102,6 +110,14 @@ ev_document_misc_get_thumbnail_frame (int        width,
 	return create_thumbnail_frame (width, height, source_pixbuf, TRUE);
 }
 
+/**
+ * ev_document_misc_get_loading_thumbnail:
+ * @width: the desired width
+ * @height: the desired height
+ * @inverted_colors: whether to invert colors
+ *
+ * Returns: (transfer full): a #GdkPixbuf
+ */
 GdkPixbuf *
 ev_document_misc_get_loading_thumbnail (int      width,
 					int      height,
@@ -143,20 +159,28 @@ ev_document_misc_paint_one_page (cairo_t      *cr,
 	GtkStyleContext *context = gtk_widget_get_style_context (widget);
 	GtkStateFlags state = gtk_widget_get_state_flags (widget);
         GdkRGBA fg, bg, shade_bg;
-        GtkSymbolicColor *c1, *c2;
 
         gtk_style_context_get_background_color (context, state, &bg);
         gtk_style_context_get_color (context, state, &fg);
-
-        // FIXME: should we cache the shade_bg?
-        c1 = gtk_symbolic_color_new_literal (&bg);
-        c2 = gtk_symbolic_color_new_shade (c1, 0.7);
-        gtk_symbolic_color_resolve (c2, NULL, &shade_bg);
-        gtk_symbolic_color_unref (c1);
-        gtk_symbolic_color_unref (c2);
+        gtk_style_context_get_color (context, state, &shade_bg);
+        shade_bg.alpha *= 0.5;
 
 	gdk_cairo_set_source_rgba (cr, highlight ? &fg : &shade_bg);
-	gdk_cairo_rectangle (cr, area);
+	cairo_rectangle (cr,
+			 area->x,
+			 area->y,
+			 area->width - border->right + border->left,
+			 area->height - border->bottom + border->top);
+	cairo_rectangle (cr,
+			 area->x + area->width - border->right,
+			 area->y + border->right - border->left,
+			 border->right,
+			 area->height - border->right + border->left);
+	cairo_rectangle (cr,
+			 area->x + border->bottom - border->top,
+			 area->y + area->height - border->bottom,
+			 area->width - border->bottom + border->top,
+			 border->bottom);
 	cairo_fill (cr);
 
 	if (inverted_colors)
@@ -169,21 +193,6 @@ ev_document_misc_paint_one_page (cairo_t      *cr,
 			 area->width - (border->left + border->right),
 			 area->height - (border->top + border->bottom));
 	cairo_fill (cr);
-
-	gdk_cairo_set_source_rgba (cr, &bg);
-	cairo_rectangle (cr,
-			 area->x,
-			 area->y + area->height - (border->bottom - border->top),
-			 border->bottom - border->top,
-			 border->bottom - border->top);
-	cairo_fill (cr);
-
-	cairo_rectangle (cr,
-			 area->x + area->width - (border->right - border->left),
-			 area->y,
-			 border->right - border->left,
-			 border->right - border->left);
-	cairo_fill (cr);
 }
 
 cairo_surface_t *
@@ -206,6 +215,12 @@ ev_document_misc_surface_from_pixbuf (GdkPixbuf *pixbuf)
 	return surface;
 }
 
+/**
+ * ev_document_misc_pixbuf_from_surface:
+ * @surface: a #cairo_surface_t
+ *
+ * Returns: (transfer full): a #GdkPixbuf
+ */
 GdkPixbuf *
 ev_document_misc_pixbuf_from_surface (cairo_surface_t *surface)
 {
@@ -379,15 +394,29 @@ ev_document_misc_invert_pixbuf (GdkPixbuf *pixbuf)
 }
 
 gdouble
-ev_document_misc_get_screen_dpi (GdkScreen *screen)
+#if GTK_CHECK_VERSION (3, 22, 0)
+ev_document_misc_get_screen_dpi (GdkScreen *screen, GdkMonitor *monitor)
+#else
+ev_document_misc_get_screen_dpi (GdkScreen *screen, gint monitor)
+#endif
 {
 	gdouble dp, di;
+	gint sc_width, sc_height;
+
+	gdk_window_get_geometry (gdk_screen_get_root_window (screen), NULL, NULL,
+				 &sc_width, &sc_height);
 
 	/*diagonal in pixels*/
-	dp = hypot (gdk_screen_get_width (screen), gdk_screen_get_height (screen));
+	dp = hypot (sc_width, sc_height);
 
 	/*diagonal in inches*/
+#if GTK_CHECK_VERSION (3, 22, 0)
+	di = hypot (gdk_monitor_get_width_mm(monitor), gdk_monitor_get_height_mm (monitor)) / 25.4;
+	di /= gdk_monitor_get_scale_factor (monitor);
+#else
 	di = hypot (gdk_screen_get_width_mm(screen), gdk_screen_get_height_mm (screen)) / 25.4;
+	di /= gdk_screen_get_monitor_scale_factor(screen, monitor);
+#endif
 
 	return (dp / di);
 }
