@@ -28,13 +28,6 @@
 #include "ev-view-marshal.h"
 #include "ev-document-misc.h"
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-#define gdk_cursor_unref g_object_unref
-#define gtk_widget_render_icon(A,B,C,D) gtk_widget_render_icon_pixbuf(A,B,C)
-#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
-#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
-#endif
-
 enum {
 	PROP_0,
 	PROP_ANNOTATION,
@@ -82,8 +75,6 @@ static guint signals[N_SIGNALS] = { 0 };
 
 G_DEFINE_TYPE (EvAnnotationWindow, ev_annotation_window, GTK_TYPE_WINDOW)
 
-#define EV_ICON_SIZE_ANNOT_WINDOW (ev_annotation_window_get_icon_size())
-
 /* Cut and paste from gtkwindow.c */
 static void
 send_focus_change (GtkWidget *widget,
@@ -105,21 +96,22 @@ send_focus_change (GtkWidget *widget,
 static gdouble
 get_screen_dpi (EvAnnotationWindow *window)
 {
-	GdkScreen *screen;
+	GdkScreen  *screen;
+#if GTK_CHECK_VERSION (3, 22, 0)
+	GdkMonitor *monitor;
+	GdkDisplay *display;
+#else
+	gint        monitor;
+#endif
 
 	screen = gtk_window_get_screen (GTK_WINDOW (window));
-	return ev_document_misc_get_screen_dpi (screen);
-}
-
-static GtkIconSize
-ev_annotation_window_get_icon_size (void)
-{
-	static GtkIconSize icon_size = 0;
-
-	if (G_UNLIKELY (icon_size == 0))
-		icon_size = gtk_icon_size_register ("ev-icon-size-annot-window", 8, 8);
-
-	return icon_size;
+#if GTK_CHECK_VERSION (3, 22, 0)
+	display = gdk_screen_get_display (screen);
+	monitor = gdk_display_get_primary_monitor (display);
+#else
+	monitor = gdk_screen_get_primary_monitor (screen);
+#endif
+	return ev_document_misc_get_screen_dpi (screen, monitor);
 }
 
 static void
@@ -139,22 +131,14 @@ ev_annotation_window_sync_contents (EvAnnotationWindow *window)
 
 static void
 ev_annotation_window_set_color (EvAnnotationWindow *window,
-				GdkColor           *color)
+                                GdkRGBA           *color)
 {
-#if GTK_CHECK_VERSION (3, 0, 0)
         GtkStyleProperties *properties;
         GtkStyleProvider   *provider;
-	GdkRGBA             rgba;
-
-        rgba.red = color->red / 65535.;
-        rgba.green = color->green / 65535.;
-        rgba.blue = color->blue / 65535.;
-        rgba.alpha = 1;
 
         properties = gtk_style_properties_new ();
         gtk_style_properties_set (properties, 0,
-                                  "color", &rgba,
-                                  "background-color", &rgba,
+                                  "background-color", color,
                                   NULL);
 
         provider = GTK_STYLE_PROVIDER (properties);
@@ -167,34 +151,14 @@ ev_annotation_window_set_color (EvAnnotationWindow *window,
         gtk_style_context_add_provider (gtk_widget_get_style_context (window->resize_sw),
                                         provider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
         g_object_unref (properties);
-#else
-	GtkRcStyle *rc_style;
-	GdkColor    gcolor;
+}
 
-	gcolor = *color;
-
-	/* Allocate these colors */
-	gdk_colormap_alloc_color (gtk_widget_get_colormap (GTK_WIDGET (window)),
-				  &gcolor, FALSE, TRUE);
-
-	/* Apply colors to style */
-	rc_style = gtk_widget_get_modifier_style (GTK_WIDGET (window));
-	rc_style->base[GTK_STATE_NORMAL] = gcolor;
-	rc_style->bg[GTK_STATE_PRELIGHT] = gcolor;
-	rc_style->bg[GTK_STATE_NORMAL] = gcolor;
-	rc_style->bg[GTK_STATE_ACTIVE] = gcolor;
-	rc_style->color_flags[GTK_STATE_PRELIGHT] = GTK_RC_BG;
-	rc_style->color_flags[GTK_STATE_NORMAL] = GTK_RC_BG | GTK_RC_BASE;
-	rc_style->color_flags[GTK_STATE_ACTIVE] = GTK_RC_BG;
-
-	/* Apply the style to the widgets */
-	g_object_ref (rc_style);
-	gtk_widget_modify_style (GTK_WIDGET (window), rc_style);
-	gtk_widget_modify_style (window->close_button, rc_style);
-	gtk_widget_modify_style (window->resize_se, rc_style);
-	gtk_widget_modify_style (window->resize_sw, rc_style);
-	g_object_unref (rc_style);
-#endif
+static void
+ev_annotation_window_set_opacity (EvAnnotationWindow *window,
+                                  gdouble             opacity)
+{
+	gtk_widget_set_opacity (GTK_WIDGET (window), opacity);
+	gtk_widget_set_opacity (GTK_WIDGET (window->text_view), opacity);
 }
 
 static void
@@ -213,10 +177,21 @@ ev_annotation_window_color_changed (EvAnnotation       *annot,
 				    GParamSpec         *pspec,
 				    EvAnnotationWindow *window)
 {
-	GdkColor color;
+	GdkRGBA rgba;
 
-	ev_annotation_get_color (annot, &color);
-	ev_annotation_window_set_color (window, &color);
+	ev_annotation_get_rgba (annot, &rgba);
+	ev_annotation_window_set_color (window, &rgba);
+}
+
+static void
+ev_annotation_window_opacity_changed (EvAnnotation       *annot,
+                                      GParamSpec         *pspec,
+                                      EvAnnotationWindow *window)
+{
+	gdouble opacity;
+
+	opacity = ev_annotation_markup_get_opacity (EV_ANNOTATION_MARKUP (annot));
+	ev_annotation_window_set_opacity (window, opacity);
 }
 
 static void
@@ -289,20 +264,20 @@ ev_annotation_window_set_resize_cursor (GtkWidget          *widget,
 						     GDK_BOTTOM_LEFT_CORNER :
 						     GDK_BOTTOM_RIGHT_CORNER);
 		gdk_window_set_cursor (gdk_window, cursor);
-		gdk_cursor_unref (cursor);
+		g_object_unref (cursor);
 	} else {
 		gdk_window_set_cursor (gdk_window, NULL);
 	}
 }
 
-static gboolean
-text_view_button_press (GtkWidget          *widget,
-			GdkEventButton     *event,
-			EvAnnotationWindow *window)
+static void
+text_view_state_flags_changed (GtkWidget     *widget,
+                               GtkStateFlags  previous_flags)
 {
-	ev_annotation_window_grab_focus (window);
+	GtkStateFlags current_flags = gtk_widget_get_state_flags (widget);
 
-	return FALSE;
+	if (current_flags & GTK_STATE_FLAG_BACKDROP)
+	    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (widget), FALSE);
 }
 
 static void
@@ -318,13 +293,17 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 	GtkWidget *vbox, *hbox;
 	GtkWidget *icon;
 	GtkWidget *swindow;
+	GtkIconTheme *icon_theme;
+	GdkPixbuf *pixbuf;
+
+	icon_theme = gtk_icon_theme_get_default ();
 
 	gtk_widget_set_can_focus (GTK_WIDGET (window), TRUE);
 
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
 	/* Title bar */
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
 	icon = gtk_image_new (); /* FIXME: use the annot icon */
 	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
@@ -340,7 +319,9 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 	g_signal_connect_swapped (window->close_button, "clicked",
 				  G_CALLBACK (ev_annotation_window_close),
 				  window);
-	icon = gtk_image_new_from_stock (EV_STOCK_CLOSE, EV_ICON_SIZE_ANNOT_WINDOW);
+	pixbuf = gtk_icon_theme_load_icon (icon_theme, EV_STOCK_CLOSE, 8, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+	icon = gtk_image_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
 	gtk_container_add (GTK_CONTAINER (window->close_button), icon);
 	gtk_widget_show (icon);
 
@@ -357,9 +338,9 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 					GTK_POLICY_AUTOMATIC);
 	window->text_view = gtk_text_view_new ();
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (window->text_view), GTK_WRAP_WORD);
-	g_signal_connect (window->text_view, "button_press_event",
-			  G_CALLBACK (text_view_button_press),
-			  window);
+	g_signal_connect (window->text_view, "state-flags-changed",
+	                  G_CALLBACK (text_view_state_flags_changed),
+	                  window);
 	gtk_container_add (GTK_CONTAINER (swindow), window->text_view);
 	gtk_widget_show (window->text_view);
 
@@ -367,7 +348,7 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 	gtk_widget_show (swindow);
 
 	/* Resize bar */
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
 	window->resize_sw = gtk_event_box_new ();
 	gtk_widget_add_events (window->resize_sw, GDK_BUTTON_PRESS_MASK);
@@ -378,7 +359,9 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 			  G_CALLBACK (ev_annotation_window_set_resize_cursor),
 			  window);
 
-	icon = gtk_image_new_from_stock (EV_STOCK_RESIZE_SW, EV_ICON_SIZE_ANNOT_WINDOW);
+	pixbuf = gtk_icon_theme_load_icon (icon_theme, EV_STOCK_RESIZE_SW, 8, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+	icon = gtk_image_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
 	gtk_container_add (GTK_CONTAINER (window->resize_sw), icon);
 	gtk_widget_show (icon);
 	gtk_box_pack_start (GTK_BOX (hbox), window->resize_sw, FALSE, FALSE, 0);
@@ -393,7 +376,9 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 			  G_CALLBACK (ev_annotation_window_set_resize_cursor),
 			  window);
 
-	icon = gtk_image_new_from_stock (EV_STOCK_RESIZE_SE, EV_ICON_SIZE_ANNOT_WINDOW);
+	pixbuf = gtk_icon_theme_load_icon (icon_theme, EV_STOCK_RESIZE_SE, 8, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+	icon = gtk_image_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
 	gtk_container_add (GTK_CONTAINER (window->resize_se), icon);
 	gtk_widget_show (icon);
 	gtk_box_pack_end (GTK_BOX (hbox), window->resize_se, FALSE, FALSE, 0);
@@ -403,17 +388,14 @@ ev_annotation_window_init (EvAnnotationWindow *window)
 	gtk_widget_show (hbox);
 
 	gtk_container_add (GTK_CONTAINER (window), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (window), 0);
 	gtk_widget_show (vbox);
 
 	gtk_widget_add_events (GTK_WIDGET (window),
 			       GDK_BUTTON_PRESS_MASK |
 			       GDK_KEY_PRESS_MASK);
-	gtk_widget_set_app_paintable (GTK_WIDGET (window), TRUE);
 
 	gtk_container_set_border_width (GTK_CONTAINER (window), 2);
 
-	gtk_window_set_accept_focus (GTK_WINDOW (window), FALSE);
 	gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
 	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window), TRUE);
 	gtk_window_set_skip_pager_hint (GTK_WINDOW (window), TRUE);
@@ -431,9 +413,10 @@ ev_annotation_window_constructor (GType                  type,
 	EvAnnotationMarkup *markup;
 	const gchar        *contents;
 	const gchar        *label;
-	GdkColor            color;
+	GdkRGBA             color;
 	EvRectangle        *rect;
 	gdouble             scale;
+	gdouble             opacity;
 
 	object = G_OBJECT_CLASS (ev_annotation_window_parent_class)->constructor (type,
 										  n_construct_properties,
@@ -457,8 +440,12 @@ ev_annotation_window_constructor (GType                  type,
 			   (gint)((rect->x2 - rect->x1) * scale),
 			   (gint)((rect->y2 - rect->y1) * scale));
 
-	ev_annotation_get_color (annot, &color);
+	ev_annotation_get_rgba (annot, &color);
 	ev_annotation_window_set_color (window, &color);
+
+	opacity = ev_annotation_markup_get_opacity (markup);
+	ev_annotation_window_set_opacity (window, opacity);
+
 	gtk_widget_set_name (GTK_WIDGET (window), ev_annotation_get_name (annot));
 	gtk_window_set_title (GTK_WINDOW (window), label);
 	gtk_label_set_text (GTK_LABEL (window->title), label);
@@ -474,9 +461,12 @@ ev_annotation_window_constructor (GType                  type,
 	g_signal_connect (annot, "notify::label",
 			  G_CALLBACK (ev_annotation_window_label_changed),
 			  window);
-	g_signal_connect (annot, "notify::color",
+	g_signal_connect (annot, "notify::rgba",
 			  G_CALLBACK (ev_annotation_window_color_changed),
 			  window);
+	g_signal_connect (annot, "notify::opacity",
+			  G_CALLBACK (ev_annotation_window_opacity_changed),
+			  window);
 
 	return object;
 }
@@ -532,6 +522,10 @@ ev_annotation_window_focus_in_event (GtkWidget     *widget,
 		window->in_move = FALSE;
 	}
 
+	gtk_widget_grab_focus (window->text_view);
+	send_focus_change (window->text_view, TRUE);
+	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (window->text_view), TRUE);
+
 	return FALSE;
 }
 
@@ -546,24 +540,6 @@ ev_annotation_window_focus_out_event (GtkWidget     *widget,
 	return FALSE;
 }
 
-static gboolean
-ev_annotation_window_enter_notify_event (GtkWidget        *widget,
-                                         GdkEventCrossing *event)
-{
-        gtk_window_set_accept_focus (GTK_WINDOW (widget), TRUE);
-
-        return FALSE;
-}
-
-static gboolean
-ev_annotation_window_leave_notify_event (GtkWidget        *widget,
-                                         GdkEventCrossing *event)
-{
-        gtk_window_set_accept_focus (GTK_WINDOW (widget), FALSE);
-
-        return FALSE;
-}
-
 static void
 ev_annotation_window_class_init (EvAnnotationWindowClass *klass)
 {
@@ -578,8 +554,6 @@ ev_annotation_window_class_init (EvAnnotationWindowClass *klass)
 	gtk_widget_class->configure_event = ev_annotation_window_configure_event;
 	gtk_widget_class->focus_in_event = ev_annotation_window_focus_in_event;
 	gtk_widget_class->focus_out_event = ev_annotation_window_focus_out_event;
-        gtk_widget_class->enter_notify_event = ev_annotation_window_enter_notify_event;
-        gtk_widget_class->leave_notify_event = ev_annotation_window_leave_notify_event;
 
 	g_object_class_install_property (g_object_class,
 					 PROP_ANNOTATION,
