@@ -61,7 +61,6 @@ typedef struct {
 typedef struct {
 	gboolean in_drag;
 	GdkPoint start;
-	gboolean in_selection;
 	GList *selections;
 	EvSelectionStyle style;
 } SelectionInfo;
@@ -178,6 +177,9 @@ struct _EvView {
 	/* Common for button press handling */
 	int pressed_button;
 
+	/* Key bindings propagation */
+	gboolean key_binding_handled;
+
 	/* Information for middle clicking and dragging around. */
 	DragInfo drag_info;
 	
@@ -189,7 +191,6 @@ struct _EvView {
 	guint selection_update_id;
 	guint selection_scroll_id;
 
-	EvViewSelectionMode selection_mode;
 	SelectionInfo selection_info;
 
 	/* Copy link address selection */
@@ -214,6 +215,15 @@ struct _EvView {
 	/* Accessibility */
 	AtkObject *accessible;
 
+	/* Caret navigation */
+	gboolean caret_enabled;
+	gint     cursor_offset;
+	gint     cursor_page;
+	gdouble  cursor_line_offset;
+	gboolean cursor_visible;
+	guint    cursor_blink_timeout_id;
+	guint    cursor_blink_time;
+
 	/* Gestures */
 	GtkGesture *pan_gesture;
 	GtkGesture *zoom_gesture;
@@ -228,23 +238,27 @@ struct _EvView {
 struct _EvViewClass {
 	GtkContainerClass parent_class;
 
-	void    (*binding_activated)	  (EvView         *view,
-					   GtkScrollType   scroll,
-					   gboolean        horizontal);
-	void    (*handle_link)		  (EvView         *view,
-					   EvLink         *link);
-	void    (*external_link)	  (EvView         *view,
-					   EvLinkAction   *action);
-	void    (*popup_menu)		  (EvView         *view,
-					   GList          *items);
-	void    (*selection_changed)      (EvView         *view);
-	void    (*sync_source)            (EvView         *view,
-					   EvSourceLink   *link);
-	void    (*annot_added)            (EvView         *view,
-					   EvAnnotation   *annot);
-	void    (*annot_removed)          (EvView         *view,
-					   EvAnnotation   *annot);
-	void    (*layers_changed)         (EvView         *view);
+	void     (*scroll)            (EvView         *view,
+	                               GtkScrollType   scroll,
+	                               GtkOrientation  orientation);
+	void     (*handle_link)       (EvView         *view,
+	                               EvLink         *link);
+	void     (*external_link)     (EvView         *view,
+	                               EvLinkAction   *action);
+	void     (*popup_menu)        (EvView         *view,
+	                               GList          *items);
+	void     (*selection_changed) (EvView         *view);
+	void     (*sync_source)       (EvView         *view,
+	                               EvSourceLink   *link);
+	void     (*annot_added)       (EvView         *view,
+	                               EvAnnotation   *annot);
+	void     (*annot_removed)     (EvView         *view,
+	                               EvAnnotation   *annot);
+	void     (*layers_changed)    (EvView         *view);
+	gboolean (*move_cursor)       (EvView         *view,
+	                               GtkMovementStep step,
+	                               gint            count,
+	                               gboolean        extend_selection);
 };
 
 void _get_page_size_for_scale_and_rotation (EvDocument *document,
@@ -270,7 +284,13 @@ void _ev_view_transform_doc_rect_to_view_rect (EvView       *view,
 					       int           page,
 					       EvRectangle  *doc_rect,
 					       GdkRectangle *view_rect);
-
+void _ev_view_get_selection_colors (EvView  *view,
+                                    GdkRGBA *bg_color,
+                                    GdkRGBA *fg_color);
+gint _ev_view_get_caret_cursor_offset_at_doc_point (EvView *view,
+                                                    gint    page,
+                                                    gdouble doc_x,
+                                                    gdouble doc_y);
 void _ev_view_set_focused_element (EvView *view,
                                    EvMapping *element_mapping,
                                    gint page);
